# Python imports
from uuid import uuid4

# Third-party imports
from neomodel import StructuredNode, StringProperty, IntegerProperty, UniqueIdProperty, RelationshipTo

# Models imports
# from api.models.dataset_model import Dataset
from api.models.ligand_solutions_model import LigandSolutions
from api.models.ligand_statistics_model import LigandStatistics
from api.models.post_refinement_cycles_model import PostRefinementCycles


class Ligand(StructuredNode):

	"""
	Defines node properties and relationships
	Provides data serializer
	"""

	# Properties
	uuid=StringProperty(unique_index=True, default=uuid4)
	ligand_id=StringProperty()

	# Relationships
	has_solutions=RelationshipTo(LigandSolutions, 'HAS')
	has_statistics=RelationshipTo(LigandStatistics, 'HAS')
	has_postrefinement_cycles=RelationshipTo(PostRefinementCycles, 'HAS')

	@property
	def serialize(self):

		"""
		Serializer for node properties
		"""
		
		return {

		'ligand_node_properties': {
		'uuid': self.uuid,
		'ligand_id': self.ligand_id,
		},
		}
